<?php

namespace App\Console\Commands;

use Illuminate\Console\Command; 
use App\Http\Controllers\ProductController;
use App\Traits\{AppConstants, MagicConstants}; 
use Illuminate\Support\Facades\{DB, Log};

class CreateProducts extends Command
{
    use MagicConstants, AppConstants;
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:create-products';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        Log::info("started => create");
        $productController =  new ProductController();
        try {
            // need to pass the connection  of common database HPVN
            // SELECT * FROM partnumberinfo where PartTerminologyID = '6832' AND BrandID='HPVN'
            // SELECT * FROM partnumberinfo where PartTerminologyID = '6832' AND BrandID='JJFD'
            // SELECT * FROM partnumberinfo where PartTerminologyID = '6832' AND BrandID='FCJP' AND SubBrandID='HPVN'
            // `PartTerminologyID`(6832) belongs to `parts` 
            $productNumberParts = DB::table('partnumberinfo')
            ->select('part_id', 'partNumber', 'PartTerminologyID')
            ->whereIn('PartTerminologyID', [PTI_CABIN_AIR_FILTER])
            ->where('BrandID', PUREFLOW_BRAND_ID)
            ->whereNotIn('partNumber', ['TBA', 'NSC', 'NRC', 'NS', 'NR']); 

            $productNumberPartsCount = $productNumberParts->count();
            $limit = 50; 
            
            Log::info("create cabin Air Filter count =>" . $productNumberPartsCount);
            $totalPage = (int)ceil($productNumberPartsCount / $limit);
            if ($productNumberPartsCount > 0) {
                $vehicleTypeGroupIDs    = $productController->getVehicleTypeGroupIDs();
                $vcdbDatabase           = config('app.pureflow_vcdb_database');
                
                // $limit = 500; // Define the limit for each batch
                $totalPage = (int)ceil($productNumberPartsCount / $limit);

                for ($i = 0; $i <= $totalPage; $i++) {
                    $productNumberPartsResult = $productNumberParts->skip($i * $limit)->take($limit)->get();

                    foreach ($productNumberPartsResult as $productNumberPart) {  
                        $applications = DB::table('applications')
                            ->select('applications.part_id', 'applications.YearID as year', 'make.MakeName as make','model.ModelName as model','applications.YearID','applications.MakeID', 'applications.ModelID', 'make.MakeName', 'model.ModelName as model', 'applications.note','applications.VehicleTypeGroupID')
                            ->leftJoin($vcdbDatabase.'.make as make', 'applications.MakeID', '=', 'make.MakeID')
                            ->leftJoin($vcdbDatabase.'.model as model', 'applications.ModelID', '=', 'model.ModelID') 
                            ->where('applications.part_id', trim($productNumberPart->part_id))
                            ->whereIn('VehicleTypeGroupID', $vehicleTypeGroupIDs)
                            ->where('MfrLabel', 'like', '%pureflow%')
                            ->get(); 
                        $applicationsCount = $applications->count();
                        $applicationTotalPage = (int)ceil($applicationsCount / $limit);
                        if($applicationsCount) {
                            for ($j = 0; $j <= $applicationTotalPage; $j++) {
                                $applicationsResult = $applications->skip($j * $limit)->take($limit);
                                if ($applicationsResult->isNotEmpty()) { 
                                    $productController->productCreateIndex($applicationsResult, $productNumberPart, 1, "create", $vehicleTypeGroupIDs);
                                }
                            } 
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $productController->errorLog($e, "Pureflow create product cron");
            report($e);
        }
        Log::info("end => create = Pureflow create product cron");
    }
}