<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use App\Http\Controllers\ProductController;
use App\Traits\{AppConstants, MagicConstants};
use Illuminate\Support\Facades\{DB, Log};

class CreateSEOProducts extends Command
{
    use MagicConstants, AppConstants;
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:create-seo-products';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        Log::info("Create SEO Products started");
        $productController = new ProductController(); 
        try {
            $currentYear = date('Y');

            $products = DB::connection('mysql2')->table('shopify_products_new')
            ->select('product_id', 'make', 'model', 'product_sku', 'year')
            ->where([
                'year'      => $currentYear,
                'is_parent' => ZERO,
            ]);
            $productNumberPartsCount = $products->count();
            $limit = 100;
            $page = ZERO;
            $totalPage = (int)ceil($productNumberPartsCount / $limit);
            if ($productNumberPartsCount > ZERO) {
                for ($i = ZERO; $i <= $totalPage; $i++) {
                    $productsResult = $products->skip($page)->take($limit)->get();
                    $page += $limit;
                    if (count($productsResult) > ZERO) {
                        foreach ($productsResult as $oldSEOProduct) {
                            $productController->createNextYearSEOProducts($oldSEOProduct);
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $productController->errorLog($e, "Pureflow Create SEO Products");
            report($e);
        }
        Log::info("end => create = Pureflow Create SEO Products");
    }
}
