<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

use App\Http\Controllers\ProductController;
use App\Traits\{AppConstants, MagicConstants};
use Illuminate\Support\Facades\{DB, Log, Mail, Http};
use App\Models\{ShopifyProduct, ShopifyProductSync, ShopifyProductMetafield};
use GuzzleHttp\Client;
use Exception;

class DeleteShopifyProductsByDate extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:delete-shopify-products-by-date';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        $date = '2024-06-06';
        $client = new Client();
        $productController = new ProductController();

        try {
            $url = $productController->getStoreUrl() . "products.json";
            $params = [
                'created_at_min' => "{$date}T00:00:00-00:00",
                'created_at_max' => "{$date}T23:59:59-00:00",
                'limit' => 250  // Shopify allows up to 250 items per page
            ];

            $products = [];
            do {
                $response = $client->get($url, [
                    'headers' => ['Content-Type' => 'application/json'],
                    'query' => $params
                ]);

                $responseData = json_decode($response->getBody()->getContents(), true);
                $products = array_merge($products, $responseData['products']);

                if (isset($responseData['links']['next'])) {
                    $url = $responseData['links']['next'];
                } else {
                    $url = null;
                }
            } while ($url);

            $this->info("Fetched " . count($products) . " products created on {$date}");

            foreach ($products as $product) {
                $this->deleteProduct($client, $product['id']);
            }

            $this->info("Products created on {$date} have been deleted.");
        } catch (Exception $e) {
            Log::error('Error fetching products from Shopify: ' . $e->getMessage());
            $this->error('An error occurred while fetching products from Shopify.');
        }
    }

    private function deleteProduct($client, $productId)
    {
        $productController =  new ProductController();

        try {
            $url        = $productController->getStoreUrl();

            $url = $url . "products/{$productId}.json";

            $response = $client->delete($url, [
                'headers' => ['Content-Type' => 'application/json']
            ]);

            if ($response->getStatusCode() == 200) {
                Log::info("Deleted product ID: {$productId}");
            } else {
                Log::warning("Failed to delete product ID: {$productId}");
            }
        } catch (\Exception $e) {
            Log::error("Error deleting product ID: {$productId}: " . $e->getMessage());
        }
    }
}