<?php

namespace App\Console\Commands\Reports;

use Illuminate\Console\Command; 
use App\Http\Controllers\ProductController;
use App\Traits\{AppConstants, MagicConstants}; 
use Illuminate\Support\Facades\{DB, Log, Mail};
use App\Mail\CreatedProductsReportMail; 

class CreateProductReport extends Command
{
    use MagicConstants, AppConstants;
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:create-product-report';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        try {
            $date = date('Y-m-d', strtotime("-1 day"));

            // Get products where emailSend is 0
            $products = DB::connection('mysql2')
            ->table('shopify_products_new')
            ->select('product_id', 'variant_id', 'product_sku', 'product_title', 'handle')
            ->where('emailSend', 0)
            ->get();

            if ($products->count()) {
                $type = "INTERNAL";
                $recipients = config("app.email_ids.created_product_report_internal");
                Mail::to($recipients)->send(new CreatedProductsReportMail($type));
            }

            // Get created product records for the previous day
            $countRecords = DB::connection('mysql2')->table('shopify_product_report')
            ->select('product_id', 'variant_id', 'product_sku', 'product_title', 'handle')
            ->join('shopify_products_new', 'shopify_products_new.product_id', "=", "shopify_product_report.shopify_product_id")
            ->where('shopify_product_report.created_at', $date)
            ->where('action', "CREATE")
            ->get();
            if ($countRecords->count()) {
                $recipients = config("app.email_ids.created_product_report_external");
                Mail::to($recipients)->send(new CreatedProductsReportMail());
            }

            $this->markProductsAsSent();
        } catch (\Exception $e) {
            $ProductController = new ProductController();
            $ProductController->errorLog($e, "create-product-report");
            report($e);
        }
    } 

    private function markProductsAsSent()
    {
        DB::connection('mysql2')
        ->table('shopify_products_new')
        ->where('emailSend', 0)
        ->update(['emailSend' => 1]);
    } 
}
