<?php

namespace App\Console\Commands\Reports;

use Illuminate\Console\Command; 
use App\Http\Controllers\ProductController; 
use Illuminate\Support\Facades\{DB, Log, Mail};
use App\Mail\DailyProductsMigrationReportMail; 


class DailyProductsMigrationReport extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:daily-products-migration-report';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'How many product created,updated,deleted (included seo product). Send Mail ';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        try { 
            $recipients = config("app.email_ids.products_migration_report");
            $date = date('Y-m-d', strtotime("-1 day"));
        
            $productsData = DB::connection('mysql2')->table('shopify_product_report')
            ->select('action', DB::raw('count(*) as total'))
            ->join('shopify_products_new', 'shopify_products_new.product_id', "=", "shopify_product_report.shopify_product_id")
            ->where('shopify_product_report.created_at', $date)->groupBy('action')
            ->get()->toArray();

            Mail::to($recipients)->send(new DailyProductsMigrationReportMail($productsData));

        } catch (\Exception $e) {
            $ProductController = new ProductController();
            $ProductController->errorLog($e, "create-report");
            report($e);
        }
    }
}
