<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Traits\{AppConstants, MagicConstants};
use Illuminate\Support\Facades\{DB, Log, Mail};
use App\Mail\ErrorDetailMail; 

class ProductController extends Controller
{
    use MagicConstants, AppConstants;
    protected $cronName = null;
    /**
     * New data dump controller
     * @param $applications
     * @param $productPartNumber
     * @param $flag
     */
    public function productCreateIndex($applications, $productPartNumber, $flag, $cronName="")
    { 
        $this->cronName = $cronName;

        try {
            foreach ($applications as $key => $part) {
                // arranged products column in single dimension array eg. [year => 2023,make => suzuki, model => vdi]
                $productPart = self::getProductType($part);
                // checking product must not exist in table.
                $shopifyProductNew = DB::connection('mysql2')->table('shopify_products_new')
                ->select('product_id')
                ->where([
                    YEAR        => $productPart[YEAR],
                    MAKE        => $productPart[MAKE],
                    MODEL       => $productPart[MODEL],
                    PRODUCT_SKU => $productPartNumber->partNumber
                    ])
                ->count();
                     
                // Product not exit, now ready for creating a new product.
                if ($shopifyProductNew <= ZERO) {
                    $nextYearSeoProduct = ZERO;
                    $shopifySeoProduct  = []; 
                    self::shopifyProductEmpty($part, $productPart, $productPartNumber, $nextYearSeoProduct, $shopifySeoProduct, $cronName);
                }

                // nextYearSeoProduct 0 => newly created, 1 => seo product created, 2=> actually products coming in next updates
                // Check product next year seo as well
                $shopifySeoProduct = DB::connection('mysql2')->table('shopify_products_new')
                    ->select('product_id')
                    ->where([
                        YEAR                => $productPart[YEAR],
                        MAKE                => $productPart[MAKE],
                        MODEL               => $productPart[MODEL],
                        PRODUCT_SKU         => $productPartNumber->partNumber,
                        'next_year_copy'    => ONE
                    ]);
                $shopifySeoProductCount = $shopifySeoProduct->count();

                if ($shopifySeoProductCount == ONE) {
                    $nextYearSeoProduct = ONE;
                    $shopifySeoProduct  = $shopifySeoProduct->first();
                    self::shopifyProductEmpty($part, $productPart, $productPartNumber, $nextYearSeoProduct, $shopifySeoProduct,$cronName);
                } 
            }
            self::emailTrigger();
        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
            Log::error("Shopify product new get set product failed");
        }
    }

    /**
     * @param $part
     * @return array
     */
    public function getProductType($part)
    {
        return [
            YEAR    => !empty($part->year) ? html_entity_decode($part->year) : "",
            MAKE    => !empty($part->make) ? html_entity_decode($part->make) : "",
            MODEL   => !empty($part->model) ? html_entity_decode($part->model) : "",
            NOTE    => $part->note ?? "",
        ];
    }

    /**
     * @param $sku
     * @return array
     */
    public function getOEMNumber($sku)
    {
        $oemNumber = [];
        $partInterchangeMapper = DB::table('partinterchange_mapper')->select('partIntPartNumber')
            ->leftJoin('partnumberinfo', 'partinterchange_mapper.part_id', '=', 'partnumberinfo.part_id')
            ->where(['partnumberinfo.partNumber' => $sku])
            ->groupby('partIntPartNumber')->get();

        if (!empty($partInterchangeMapper)) {
            foreach ($partInterchangeMapper as $partInterchange) {
                $oemNumber[] = $partInterchange->partIntPartNumber;
            }
        }

        return $oemNumber;
    }

    /**
     * @param $part
     * @param $productPart
     * @param $productPartNumber
     * @param $nextYearSeoProduct
     * @param $shopifySeoProduct
     * @return false|int|string
     */

    public function shopifyProductEmpty($part, $productPart, $productPartNumber, $nextYearSeoProduct, $shopifySeoProduct,$cronName)
    {
        try {
            $sku                = $productPartNumber->partNumber;
            $product            = [];
            $sizeVariant        = self::getPackages($sku);
            $product            = self::productImages($sku, $product); // set image url
            $oemNumber          = self::getOEMNumber($sku);
            $technicalDetails   = self::getTechnicalDetail($sizeVariant, $sku, $oemNumber);

            if ($nextYearSeoProduct == ZERO) {
                $product = self::metaFields($product, $sizeVariant, $technicalDetails, $productPart, $productPartNumber, ZERO, $part); // product metafields is being prepare
            }

            $product = self::basicProductDetails($product, $productPart, $sku, $sizeVariant[WEIGHT], $part, $productPartNumber); // set product basic keys

            if ($nextYearSeoProduct == ONE) {
                $product                    = self::seoProductUpdate($product, $part, $productPart, $shopifySeoProduct, $productPartNumber);
                $pro['product']             = $product; // final product array is ready
                $data_json                  = json_encode($pro, JSON_HEX_TAG);
                $url                        = self::getStoreUrl();
                $url                        = $url . "products/" . $shopifySeoProduct->product_id . ".json";
                // Log::info(["1", $url, $data_json]);
                $shopifyResponse            = self::curlRequest($url, $data_json, "PUT"); // send post request using
                $product['next_year_copy']  = ZERO;
            }

            if ($nextYearSeoProduct == ZERO) {
                $pro['product']             = $product; // final product array is ready
                $data_json                  = json_encode($pro, JSON_HEX_TAG);
                $url                        = self::getStoreUrl();
                $url                        = $url . "products.json";
                // Log::info(["2",$url, $data_json]);
                $shopifyResponse            = self::curlRequest($url, $data_json, "POST"); // send post request using
                $product['next_year_copy']  = ZERO;
            }

            $data_insert = json_encode($shopifyResponse, JSON_HEX_TAG);
            DB::connection('mysql2')->table('shopify_log_details')->insert([
                'purl'          => $url,
                'method'        => 'NEW PRODUCT CREATION',
                'json_data'     => $data_json,
                'response_data' => $data_insert,
            ]);

            if (!array_key_exists('errors', $shopifyResponse) && !array_key_exists('error', $shopifyResponse)) {

                if ($nextYearSeoProduct == ONE) {
                    $action = 'NEXT_YEAR_SEO_UPDATE';
                } else if ($nextYearSeoProduct == ZERO) {
                    $action = 'CREATE';
                }

                DB::connection('mysql2')->table('shopify_product_report')->insert([
                    'action'                => $action,
                    'shopify_product_id'    => $shopifyResponse['product']['id'],
                    'cron_name'             => $cronName,
                    'created_at'            => date('Y-m-d'),
                ]);

                $imagesrc = $image_ids = [];
                foreach ($shopifyResponse['product']['images'] as $image) {
                    list($filename, $Vtime) = explode("?", basename($image['src']));
                    $imagesrc[]             = $filename;
                    $image_ids[]            = $image['id'];
                }
                $shopifyResponse['product']['src']      = implode(',', $imagesrc);
                $shopifyResponse['product']['img_ids']  = implode(',', $image_ids);
                $tagCheck                               = explode(',', $shopifyResponse['product']['tags']);
                $trimmed_tags                           = array_map('trim', $tagCheck);

               $createdProductID = self::createProduct($shopifyResponse, $sku, $technicalDetails, $trimmed_tags, $productPart, $product, $productPartNumber);
                if ($nextYearSeoProduct == ONE) {
                    $shopifySeoProduct = DB::connection('mysql2')->table('shopify_products_new')
                        ->where('product_id', $shopifyResponse['product']['id'])->first();
                    //self::updateProduct($shopifySeoProduct);
                }
                if ($nextYearSeoProduct == ZERO) {
                    self::getMetaFields($shopifyResponse['product']['id'], $productPartNumber);
                }
            } else {
                Log::info("Product Create");
                Log::error($data_insert);
                self::emailTriggerOnError($data_insert);
            }
            return $data_insert;
        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
            Log::info("product Create failed.");
            self::emailTriggerOnError($e->getMessage());
        }
    }

    public function getVehicleTypeGroupIDs() {
        // Define the array of VehicleTypeIDs
        $acceptedVehicleTypeIDs = [5, 1000157964, 2187, 1000157990, 6, 7, 1000157996];

        // Retrieve the distinct VehicleTypeGroupIDs and convert to array
        $vehicleTypeGroupIDs = DB::connection('mysql1')
            ->table('vehicletype')
            ->distinct()
            ->whereIn('VehicleTypeID', $acceptedVehicleTypeIDs)
            ->pluck('VehicleTypeGroupID')
            ->toArray();

        return $vehicleTypeGroupIDs;
    }
    /**
     * @param $product
     * @param $productPart
     * @param $sku
     * @param $weight
     * @param $part
     * @return mixed
     */
    public function basicProductDetails($product, $productPart, $sku, $weight, $part, $productPartNumber)
    {
        $tags                       = [];
        $vehicle_type               = $fitment = $product_type = "";
        $product['vendor']          = 'PUREFLOW AIR';
        $product['product_type']    = 'Default';
        $product['status']          = 'draft';
        

        // $product['vendor']  = PREMIUM_GUARD;
        $product['brand']   = PREMIUM_GUARD; 
        if ($product['hx'] == 'HX') {
            $title  = 'Cabin Air Filter HEPA';
            $handle = '-cabin-air-filter-hepa-';
            $tags[] = "hepa";
        } else {
            if ($sku == 'PC4313XK' || $sku == 'PC6205XK' || $sku == 'PC9957XK') {
                $title  = 'Cabin Air Filter and Access Door Kit';
                $handle = '-cabin-air-filter-and-access-door-kit-';
            } else {
                $title  = 'Cabin Air Filter';
                $handle = '-cabin-air-filter-';
            }
            $tags[]     = "non-hepa";
        }
        $product_type       = 'cabin_air_filter';
        $tags[]             = $product_type;
        $tags[]             = "Fitment_cabin_air_filter";
        $tags[]             = "Fitment_cabin_air_filter";
        $product['vendor']  = "PUREFLOW AIR"; 

        unset($product['hx']);

        if (!empty($part->vehicleTypeGroupId)) {
            $tags[] = 'vehicle_type_' . $part->vehicleTypeGroupId;
            $tags[] = "vehicle_group_type_" . $part->vehicleTypeGroupId;
        }

        $product['title']   = $productPart[YEAR] . ' ' . $productPart[MAKE] . ' ' . $productPart[MODEL] . ' ' . $title . ' ' . $sku;
        $product['handle']  = strtolower($productPart[YEAR]) . '-' . strtolower($productPart[MAKE]) . '-' . strtolower($productPart[MODEL]) . $handle . '' . strtolower($sku);
        $tags[]             = "Category_Default Category/default";
        $tags[]             = "Year_" . $productPart[YEAR];
        $tags[]             = "Make_" . $productPart[MAKE];
        $tags[]             = "Model_" . $productPart[MODEL];
        $tags[]             = "YGroup_" . $sku; 
        $tags[]             = $sku;
        // Log::info($product);
        // Log::info($tags);
        // $tags[] = isset($product['tags']) ? $product['tags']:'';
        $product['tags']            = implode(",", $tags);
        $product['product_type']    = $product_type;
        $product['published']       = 'false';

        if ($weight == ZERO) {
            $product['variants'] = array(array('title' => 'Default Title', 'sku' => $sku, 'inventory_management' => 'shopify'));
        } else {
            $product['variants'] = array(array('title' => 'Default Title', 'sku' => $sku, WEIGHT => $weight, 'inventory_management' => 'shopify',));
        }
        return $product;
    }

    /**
     * @param $sku
     * @param $product
     * @return mixed
     */
    public function productImages($sku, $product)
    {
        $product[IMAGES] = [];
        $imageMappers = DB::table('image_mapper')->select('fileName')
            ->leftJoin('partnumberinfo', 'image_mapper.part_id', '=', 'partnumberinfo.part_id')
            ->where([
            'partnumberinfo.partNumber' => $sku,
        ])
        ->orderByRaw("CASE WHEN AssetType = 'p04' THEN 1 ELSE 0 END ASC")
        ->orderBy(ASSETTYPE, 'DESC')
        ->orderByRaw("SUBSTRING(fileName,INSTR(fileName, '-')+1, 1)")->get()->toArray();
        $attachments    = [];
        $is_hepa        = substr($sku, -2);
        $product['hx']  = $is_hepa;
        $key            = '';
        if (count($imageMappers) > ZERO) {
            if ($is_hepa == 'HX') {
                $hepaWords = 'Hepa leading image';
                foreach ($imageMappers as $ikey => $imageMapper) {
                    if (strpos($imageMapper->fileName, $hepaWords) !== false) {
                        $key = $ikey;
                    }
                }
                if (!empty($key)) {
                    $moveposition = $imageMappers[$key];
                    unset($imageMappers[$key]);
                    array_unshift($imageMappers, $moveposition);
                }
                $image_postion = ONE;
                foreach ($imageMappers as $ik => $ival) {
                    $image_path     = "https://images.pgfilters.com/assets/" . $ival->fileName;
                    $attachments[]  = array('src' => $image_path, "position" => $image_postion);
                    $image_postion++;
                }
            } else {                 
                $hepaWords = 'Pureflow leading image';
                foreach ($imageMappers as $ikey => $imageMapper) {
                    if (strpos($imageMapper->fileName, $hepaWords) !== false) {
                        $key = $ikey;
                    }
                }
                if (!empty($key)) {
                    $moveposition = $imageMappers[$key];
                    unset($imageMappers[$key]);
                    array_unshift($imageMappers, $moveposition);
                }
                $image_position = ONE;
                foreach ($imageMappers as $ik => $ival) {
                    $image_path     = "https://images.pgfilters.com/assets/" . $ival->fileName;
                    $attachments[]  = array('src' => $image_path, "position" => $image_position);
                    $image_position++;
                }
            }
        }
        if (!empty($attachments)) {
            $product[IMAGES] = $attachments;
        }
        return $product;
    }

    /**
     * @param $product
     * @param $sizeVariant
     * @param $technicalDetails
     * @param $productPart
     * @return mixed
     */
    public function metaFields($product, $sizeVariant, $technicalDetails, $productPart, $productPartNumber, $edit = ZERO, $part)
    {
        $metaFields = [];
        $interchangeName = DB::table('partinterchange_mapper')
            ->select('BrandName as interchange_name', 'partIntPartNumber')
            ->leftJoin('interchangename_brandcode_mapper', "partinterchange_mapper.partIntBrandID", "=", "interchangename_brandcode_mapper.BrandID")
            ->where(["partinterchange_mapper.part_id" => $productPartNumber->part_id])
            //->groupBy('interchange_name')
            ->get()->toArray();
        $competitors = [];
        foreach ($interchangeName as $ts) {
            $competitors[$ts->interchange_name] = $ts->partIntPartNumber;
        }

        if (!empty($competitors)) {
            $metaFields[] = array(
                "key"       => 'competitors_interchanges',
                "value"     => json_encode($competitors, JSON_HEX_TAG),
                "type"      => "json",
                "namespace" => "custom"
            );
        }

        $buyersArray = DB::table('part_attributes_mapper')
            ->select('PAName', 'value')
            ->leftJoin('part_attributes_master', "part_attributes_mapper.PAID", "=", "part_attributes_master.PAID")
            ->where(["part_attributes_mapper.part_id" => $productPartNumber->part_id])
            ->get()
            //->groupBy('PAName')
            ->toArray();
        $buyers = [];
        foreach ($buyersArray as $bt) {
            //foreach ($bt as $b){
            $buyers[$bt->PAName] = $bt->value;
            //}
        }
        $buyers = array_map('utf8_encode', $buyers);
        if (!empty($buyers)) {
            $metaFields[] = array(
                "key"       => 'buyers_guide',
                "value"     => json_encode($buyers, JSON_HEX_TAG),
                "type"      => "json",
                "namespace" => "custom"
            );
        }

        if ($edit == ONE) {
            $vcdbDatabase = config('app.pureflow_vcdb_database');
            $applications = DB::table('applications as tab1')
                ->select('tab1.part_id', 'tab1.YearID as year', 'make.MakeName as make', 'model.ModelName as model', 'tab1.YearID', 'tab1.MakeID', 'tab1.ModelID', 'tab1.note', 'tab1.VehicleTypeGroupID')
                ->leftJoin($vcdbDatabase . '.make as make', 'tab1.MakeID', '=', 'make.MakeID')
                ->leftJoin($vcdbDatabase . '.model as model', 'tab1.ModelID', '=', 'model.ModelID')
                ->where('tab1.part_id', trim($productPartNumber->part_id));

            if (isset($productPart['year'])) {
                $applications->where('tab1.YearID', trim($productPart['year']));
            }

            if (isset($productPart['make'])) {
                $applications->where('make.MakeName', trim($productPart['make']));
            }

            if (isset($productPart['model'])) {
                $applications->where('model.ModelName', trim($productPart['model']));
            }

            $applications = $applications->get()->toArray();

            // $applications = DB::table('applications')
            //     ->select('part_id', 'year', 'make', 'model', 'note', 'vehicleTypeGroupId')
            //     ->where('part_id', trim($productPartNumber->part_id))
            //     ->where('year', trim($productPart['year']))
            //     ->where('make', trim($productPart['make']))
            //     ->where('model', trim($productPart['model']))
            //     ->get()->toArray();
            if (!empty($applications)) {
                if (!empty($applications[ZERO]->note)) {
                    $metaFields[] = array(
                        KEY         => 'qualifier',
                        VALUE       => $applications[ZERO]->note,
                        TYPE        => "multi_line_text_field",
                        NAMESPACES  => "global"
                    );
                }

            }
        } else {
            if (!empty($productPart[NOTE])) {
                $metaFields[] = array(
                    KEY         => 'qualifier',
                    VALUE       => $productPart[NOTE],
                    TYPE        => "multi_line_text_field",
                    NAMESPACES  => "global"
                );
            }
        }

        /*$metaFields[] = array(
            KEY => 'pdf',
            VALUE => '',
            VALUE_TYPE => "string",
            NAMESPACES => "custom"
        );*/

        $metaFields[] = array(
            "key"       => 'technical_details',
            "value"     => json_encode($technicalDetails, JSON_HEX_TAG),
            "type"      => "string",
            "namespace" => "global"
        );

        $metaFields[] = array(
            "key"       => 'brand',
            "value"     => PREMIUM_GUARD,
            TYPE        => "single_line_text_field",
            "namespace" => "custom"
        );

        $metaFields[] = array(
            "key"       => 'sku',
            "value"     => $productPartNumber->partNumber,
            TYPE        => "single_line_text_field",
            "namespace" => "global"
        );

        $metaFields[] = array(
            "key"       => 'year',
            "value"     => $productPart[YEAR],
            TYPE        => "single_line_text_field",
            "namespace" => "global"
        );

        $metaFields[] = array(
            "key"       => 'make',
            "value"     => $productPart[MAKE],
            TYPE        => "single_line_text_field",
            "namespace" => "global"
        );

        $metaFields[] = array(
            "key"       => 'model',
            "value"     =>  $productPart[MODEL],
            TYPE        => "single_line_text_field",
            "namespace" => "global"
        );

        if (!empty($sizeVariant[WEIGHT])) {
            $metaFields[] = array(
                KEY         => WEIGHT,
                VALUE       => $sizeVariant[WEIGHT],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }
        if (!empty($sizeVariant[LENGTH])) {
            $metaFields[] = array(
                KEY         => LENGTH,
                VALUE       => $sizeVariant[LENGTH],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }
        if (!empty($sizeVariant[WIDTH])) {
            $metaFields[] = array(
                KEY         => WIDTH,
                VALUE       => $sizeVariant[WIDTH],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }
        if (!empty($sizeVariant[HEIGHT])) {
            $metaFields[] = array(
                KEY         => HEIGHT,
                VALUE       => $sizeVariant[HEIGHT],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }
        if (!empty($sizeVariant[DIMENSIONS]) && $sizeVariant[DIMENSIONS] != "0,0,0") {
            $metaFields[] = array(
                KEY         => DIMENSIONS,
                VALUE       => $technicalDetails[DIMENSIONS],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }
        if (!empty($technicalDetails[OEM_PART_NUMBER])) {
            $metaFields[] = array(
                KEY         => OEM_PART_NUMBER,
                VALUE       => $technicalDetails[OEM_PART_NUMBER],
                TYPE        => "single_line_text_field",
                NAMESPACES  => "custom"
            );
        }

        $vehicleTypeGroupIDs    = $this->getVehicleTypeGroupIDs();
        $vcdbDatabase           = config('app.pureflow_vcdb_database');
        $RegionNames            = []; 
        if ($edit == ONE) {
            $RegionNames = DB::table('applications as tab1')
            ->leftJoin($vcdbDatabase . '.region as tab2', 'tab1.RegionID', '=', 'tab2.RegionID')
            ->leftJoin($vcdbDatabase . '.make as make', 'tab1.MakeID', '=', 'make.MakeID')
            ->leftJoin($vcdbDatabase . '.model as model', 'tab1.ModelID', '=', 'model.ModelID')
            ->where('tab1.YearID', $part->year)
            ->where('make.MakeName', $part->make)
            ->where('model.ModelName', $part->model)
            ->where('tab1.part_id', $productPartNumber->part_id)
            ->whereIn('tab1.VehicleTypeGroupID', $vehicleTypeGroupIDs)
            ->select('tab2.RegionName')
            ->get()
            ->pluck('RegionName')
            ->toArray();

        } else {
            $RegionNames = DB::table('applications as tab1')
            ->leftJoin($vcdbDatabase . '.region as tab2', 'tab1.RegionID', '=', 'tab2.RegionID')
            ->where('tab1.YearID', $part->YearID)
            ->where('tab1.MakeID', $part->MakeID)
            ->where('tab1.ModelID', $part->ModelID)
            ->where('tab1.part_id', $productPartNumber->part_id)
            ->whereIn('tab1.VehicleTypeGroupID', $vehicleTypeGroupIDs)
            ->select('tab2.RegionName')
            ->get()
            ->pluck('RegionName')
            ->toArray();
        }
 

        if (!empty($RegionNames)) { 
            $metaFields[] = [
                "key"       => 'region',
                "value"     => implode(', ', $RegionNames),
                "type"      => "string",
                "namespace" => "global"
            ]; 
            Log::info($metaFields);
        }

        if (!empty($metaFields)) {
            $product['metafields'] = $metaFields;
        }
        return $product;
    }

    /**
     * @param $shopifyResponse
     * @param $sku
     * @param $technicalDetails
     * @param $trimmed_tags
     * @param $productPart
     * @param $product
     * @param $productPartNumber
     * @return bool
     */
    public function createProduct($shopifyResponse, $sku, $technicalDetails, $trimmed_tags, $productPart, $product, $productPartNumber)
    {
        return DB::connection('mysql2')
                ->table('shopify_products_new')
                ->updateOrInsert([
                    'product_id' => $shopifyResponse['product']['id'],
                //'variant_id' => $shopifyResponse['product']['variants'][0]['id'],
                //'product_sku' => $sku
            ], [
            'product_id'            => $shopifyResponse['product']['id'],
            'variant_id'            => $shopifyResponse['product']['variants'][0]['id'],
            'inventory_item_id'     => $shopifyResponse['product']['variants'][0]['inventory_item_id'],
            'product_title'         => $shopifyResponse['product']['title'],
            'handle'                => $shopifyResponse['product']['handle'],
            'product_sku'           => $sku,
            'technical_details'     => json_encode($technicalDetails, JSON_HEX_TAG),
            'tags'                  => json_encode($trimmed_tags),
            'images'                => $shopifyResponse['product']['src'] ?? "",
            'image_ids'             => $shopifyResponse['product']['img_ids'] ?? "",
            'status'                => ZERO,
            'year'                  => $productPart['year'],
            'make'                  => $productPart['make'],
            'model'                 => $productPart['model'],
            'engine'                => '',
            'region'                => '',
            'inventory_management'  => '1',
            'is_new'                => '1',
            'is_new_copy'           => $product['is_new_copy'] ?? ZERO,
        ]);
        // 'product_type'          => $product['product_type'],
        // 'filter_search'         => self::filterType($productPartNumber),
    }

    /**
     * @param $sku
     * @return array
     */
    public function getPackages($sku)
    {
        $sizeVariant    = [];
        $packages       = DB::table('package')
                            ->leftJoin('partnumberinfo', 'package.part_id', '=', 'partnumberinfo.part_id')
                            ->where(['partnumberinfo.partNumber' => $sku])
                            ->first();

        $sizeVariant = [
            HEIGHT  => ZERO,
            WIDTH   => ZERO,
            LENGTH  => ZERO,
            WEIGHT  => ZERO,
        ];

        if ($packages) {
            $sizeVariant[HEIGHT]    = $packages->height;
            $sizeVariant[WIDTH]     = $packages->width;
            $sizeVariant[LENGTH]    = $packages->length;
            $sizeVariant[WEIGHT]    = $packages->weight;
        }
        return $sizeVariant;
    }

    /**
     * @param $sizeVariant
     * @param $sku
     * @param $oemNumber
     * @return array
     */
    public function getTechnicalDetail($sizeVariant, $sku, $oemNumber)
    {
        return [
            BRAND           => PREMIUM_GUARD,
            WEIGHT          => $sizeVariant[WEIGHT],
            DIMENSIONS      => $sizeVariant[LENGTH] . ',' . $sizeVariant[WIDTH] . ',' . $sizeVariant[HEIGHT],
            PART_NUMBER_C   => $sku,
            OEM_PART_NUMBER => implode(',', $oemNumber)
        ];
    }

    /**
     * @param $shopifyProductOldRow
     * @param $flg
     */
    public function updateProduct_og($shopifyProductOldRow, $flg = ZERO,$cronName = "")
    {
        try {
            $sku            = $shopifyProductOldRow->product_sku;
            // $product_type   = self::getProductTypeId($shopifyProductOldRow->product_type);
            $productId      = $shopifyProductOldRow->product_id;
            $metaArray      = self::isMetaFieldsExist($productId);
            $metaArray      = json_decode(json_encode($metaArray), true);

            $url            = self::getStoreUrl();
            $productURL     = $url . "products/$productId";
            // metafield update or remove if removed from database dump
            if (empty($metaArray)) {
                $metaArray1 = self::getMetaFields($productId);
                $metaArray  = isset($metaArray1['metafields']) ? $metaArray1['metafields'] : [];
            } else {
                $url        = $url . "products/$productId/metafields.json";
                $metaArray1 = self::curlRequest($url, "", "GET");
                $metaArray  = isset($metaArray1['metafields']) ? $metaArray1['metafields'] : [];
            }

            $product                        = $sizeVariant = $oemNumber = $technicalDetails = [];
            $productPart                    = self::getProductType($shopifyProductOldRow);
            $oemNumber                      = self::getOEMNumber($sku);
            $sizeVariant                    = self::getPackages($sku);
            $technicalDetails               = self::getTechnicalDetail($sizeVariant, $sku, $oemNumber);
            $productNumberPart              = new \stdClass();
            $productNumberPart->partNumber  = $sku;
            $preparedMetafields             = self::metaFields($product, $sizeVariant, $technicalDetails, $productPart, $productNumberPart, ONE, $shopifyProductOldRow);
            foreach ($preparedMetafields['metafields'] as $jKey => $preparedMetafield) {
                foreach ($metaArray as $key => $meta) {
                    if ($preparedMetafield[KEY] == $meta[KEY] && $preparedMetafield[VALUE] == $meta[VALUE]) {
                        unset($metaArray[$key]);
                        unset($preparedMetafields['metafields'][$jKey]);
                        break;
                    }
                }
            }
            // deleted old metafield which is invalid after updated dump
            if (!empty($metaArray)) {
                $metaArray = array_values($metaArray);

                foreach ($metaArray as $pm) {
                    $metaId         = isset($pm['metafield_id']) ? $pm['metafield_id'] : $pm['id'];
                    $metaurl        = $productURL . "/metafields/" . $metaId . ".json";
                    $metafields     = self::curlRequest($metaurl, "", "DELETE");
                    $data_insert    = json_encode($metafields, JSON_HEX_TAG);

                    DB::connection('mysql2')->table('shopify_log_details')->insert([
                        'purl'          => $metaurl,
                        'method'        => 'DELETE METAS',
                        'json_data'     => json_encode($metaId, JSON_HEX_TAG),
                        'response_data' => $data_insert,
                    ]);
                    DB::connection('mysql2')->table('shopify_products_metafiels')
                        ->where([ID => $pm[ID]])
                        ->delete();
                }
            }
            //dd($preparedMetafields['metafields']);
            // insert new metafield after updated dump
            if (!empty($preparedMetafields['metafields'])) {
                $preparedMetafields['metafields']   = array_values($preparedMetafields['metafields']);
                $productCreated                     = [];
                $productCreated['id']               = $productId;
                $productCreated['metafields']       = $preparedMetafields['metafields'];

                $data_json      = json_encode(['product' => $productCreated], JSON_HEX_TAG);
                $metafields     = self::curlRequest($productURL . ".json", $data_json, "PUT");
                $data_insert    = json_encode($metafields, JSON_HEX_TAG);
                DB::connection('mysql2')->table('shopify_log_details')->insert([
                    'purl'          => $productURL . ".json",
                    'method'        => 'Insert update METAS',
                    'json_data'     => $data_json,
                    'response_data' => $data_insert,
                ]);
                self::getMetaFields($productId);
            }

            $imageids = explode(",", $shopifyProductOldRow->image_ids);
            if (count($imageids) <= 0) {
                $purl                   = $productURL . ".json";
                $productArray           = self::curlRequest($purl, "", "GET");
                $shopifyProductOldRow   = self::imageUpdateToDatabase($productArray, 1, $productId);
            }

            $imageids = explode(",", $shopifyProductOldRow->image_ids);
            if (count($imageids) > 0) {
                $productImages      = self::productImages($sku, $product);
                $UpdatedImageCount  = isset($productImages['images']) ? count($productImages['images']) : ZERO;
                if (count($imageids) != $UpdatedImageCount) {
                    foreach ($imageids as $pid) {
                        $pdeleteurl     = $productURL . "/images/" . $pid . ".json";
                        $imageResult    = self::curlRequest($pdeleteurl, "", "DELETE");
                        $data_insert    = json_encode($imageResult, JSON_HEX_TAG);
                        DB::connection('mysql2')->table('shopify_log_details')->insert([
                            'purl'          => $pdeleteurl,
                            'method'        => 'DELETE Images',
                            'json_data'     => json_encode($pid, JSON_HEX_TAG),
                            'response_data' => $data_insert,
                        ]);
                    }
                    self::imageDeleteFromDatabase($productId);

                    $productCreated             = [];
                    $productCreated['id']       = $productId;
                    $productCreated['images']   = $productImages['images'];
                    $data_json                  = json_encode(['product' => $productCreated], JSON_HEX_TAG);
                    $image_update_url           = $productURL . ".json";
                    $pImages                    = self::curlRequest($image_update_url, $data_json, "PUT");
                    $data_insert                = json_encode($pImages, JSON_HEX_TAG);
                    DB::connection('mysql2')->table('shopify_log_details')->insert([
                        'purl'          => $image_update_url,
                        'method'        => 'Image updated while',
                        'json_data'     => $data_json,
                        'response_data' => $data_insert,
                    ]);
                    self::imageUpdateToDatabase($pImages, 0, $productId);
                }
            }
            DB::connection('mysql2')->table('shopify_product_report')->insert([
                'action'                => 'UPDATE',
                'shopify_product_id'    => $productId,
                'cron_name'             => $cronName,
                'created_at'            => date('Y-m-d'),
            ]);
        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
            Log::error("Edit error - updateProduct_og()");
        }
    }

    /**
     * @param $shopifyProductOldRow
     * @param int $flg
     * @param string $cronName
     */
    public function updateProduct($shopifyProductOldRow, $flg = ZERO, $cronName = "")
    {
        try { 
            $sku            = $shopifyProductOldRow->product_sku; 
            $productId      = $shopifyProductOldRow->product_id;

            // To get meta from db
            $metaDestinationDBArray = self::isMetaFieldsExist($productId); // if not meta found in destination database
            $metaDestinationDBArray = json_decode(json_encode($metaDestinationDBArray), true);
            $metaDestinationDBArray = array_map("unserialize", array_unique(array_map("serialize", $metaDestinationDBArray)));

            // To get meta from source database
            $product = $sizeVariant = $oemNumber = $technicalDetails = [];
            $productPart                    = self::getProductType($shopifyProductOldRow);
            $oemNumber                      = self::getOEMNumber($sku);
            $sizeVariant                    = self::getPackages($sku);
            $technicalDetails               = self::getTechnicalDetail($sizeVariant, $sku, $oemNumber);
            $productNumberPart              = new \stdClass();
            $productNumberPart->part_id     = $sku;
            $productNumberPart->partNumber  = $sku;
            $preparedFromSourceDBMetafields = self::metaFields($product, $sizeVariant, $technicalDetails, $productPart, $productNumberPart, ONE, $shopifyProductOldRow);

            $flgUpdate = $flgInsert = $flgDelete = $flgImage = 0;
            // get different meta from source db and destination db meta start
            //dump($preparedFromSourceDBMetafields,$metaDestinationDBArray);
            foreach ($preparedFromSourceDBMetafields['metafields'] as $ikey => $preparedMetafield) {
                foreach ($metaDestinationDBArray as $key => $meta) {
                    $metaValue  = $preparedMetafield[VALUE];
                    $metaD      = $meta[VALUE];

                    if ($preparedMetafield[KEY] == 'buyers_guide' || $preparedMetafield[KEY] == 'competitors_interchanges') {
                        $metaValue = json_decode($preparedMetafield[VALUE], true);
                    }
                    if ($meta[KEY] == 'buyers_guide' || $meta[KEY] == 'competitors_interchanges') {
                        $metaD = json_decode($meta[VALUE], true);
                    }

                    if ($preparedMetafield[KEY] == $meta[KEY] && !empty($metaValue) && !empty($metaD)) {
                        unset($preparedFromSourceDBMetafields['metafields'][$ikey]);
                        unset($metaDestinationDBArray[$key]);
                        break;
                    }
                }
            }
            // get different meta from source db and destination db meta end

            // if we get different then let's post, put and delete

            $url        = self::getStoreUrl();
            $productURL = $url . "products/$productId";
            // Let's find meta and if we get different then delete from Shopify store and Destination DB
            if (!empty($metaDestinationDBArray)) {
                $url                = $url . "products/$productId/metafields.json";
                // Log::info(['dd', $url]);

                $metaShopifyArray   = self::curlRequest($url, "", "GET"); // get all meta by product id from store
                $metaShopifyArray   = isset($metaShopifyArray['metafields']) ? $metaShopifyArray['metafields'] : [];

                // destination meta delete from db and Shopify which were not match to source db meta
                foreach ($metaDestinationDBArray as $ikey => $metaDestinationDB) {
                    if (!empty($metaShopifyArray)) {
                        foreach ($metaShopifyArray as $metaShopify) {
                            $metaValue = $metaDestinationDB[VALUE];
                            $metaD = $metaShopify[VALUE];
                            if ($metaDestinationDB[KEY] == 'buyers_guide' || $metaDestinationDB[KEY] == 'competitors_interchanges') {
                                $metaValue = json_decode($preparedMetafield[VALUE], true);
                            }

                            if ($metaShopify[KEY] == 'buyers_guide' || $metaShopify[KEY] == 'competitors_interchanges') {
                                $metaD = json_decode($meta[VALUE], true);
                            }

                            if ($preparedMetafield[KEY] == $meta[KEY] && !empty($metaValue) && !empty($metaD)) {
                                $metaDestinationDBRows = DB::connection('mysql2')->table('shopify_products_metafiels')->where(
                                    [
                                        'product_id'    => $metaDestinationDB['product_id'],
                                        'namespace'     => $metaDestinationDB['namespace'],
                                        'key'           => $metaDestinationDB['key'],
                                        'value'         => $metaDestinationDB['value'],
                                    ]);
                                $metaDestinationDBRow = $metaDestinationDBRows->first();

                                // Log::info([$metaDestinationDBRow, $metaDestinationDBRows]); 

                                if ($metaDestinationDBRow) {
                                    $flgDelete      = 1;
                                    $metafields_id  = $metaDestinationDBRow->metafield_id;
                                    $metaurl        = $productURL . "/metafields/" . $metafields_id . ".json";
                                    $metafields     = self::curlRequest($metaurl, "", "DELETE");
                                    $data_insert    = json_encode($metafields, JSON_HEX_TAG);

                                    DB::connection('mysql2')->table('shopify_log_details')->insert([
                                        'purl'          => $metaurl,
                                        'method'        => 'DELETE METAS',
                                        'json_data'     => json_encode($metaDestinationDBRow->metafield_id, JSON_HEX_TAG),
                                        'response_data' => $data_insert,
                                    ]);
                                    $metaDestinationDBRows->delete();
                                }
                            }
                        }

                    }
                    if(!empty($metaDestinationDBArray)){
                        foreach ($metaDestinationDBArray as  $ikey => $metaDestinationDB) {
                            $metaDestinationDBRows = DB::connection('mysql2')->table('shopify_products_metafiels')->where(
                                [
                                    'product_id'    => $metaDestinationDB['product_id'],
                                    'namespace'     => $metaDestinationDB['namespace'],
                                    'key'           => $metaDestinationDB['key'],
                                    'value'         => $metaDestinationDB['value'],
                                ]);
                            $metaDestinationDBRows->delete();
                        }
                    }

                } 
            }
            // dd($preparedFromSourceDBMetafields['metafields']);

            // if meta exist on shopify then update or insert new meta save to db
            if (!empty($preparedFromSourceDBMetafields['metafields'])) {

                $url                = self::getStoreUrl();
                $url                = $url . "products/$productId/metafields.json";
                $metaShopifyArray   = self::curlRequest($url, "", "GET"); // get all meta from store
                $metaShopifyArray   = isset($metaShopifyArray['metafields']) ? $metaShopifyArray['metafields'] : [];

                // for update meta on store and db
                foreach ($preparedFromSourceDBMetafields['metafields'] as $ikey => $preparedFromSourceDB) {
                    if (!empty($metaShopifyArray)) {
                        foreach ($metaShopifyArray as $metaShopify) {
                            if ($preparedFromSourceDB[KEY] == $metaShopify[KEY]) {
                                $productMetafieldsCreated               = [];
                                $productMetafieldsCreated['id']         = $productId;
                                $productMetafieldsCreated['metafields'] = [$preparedFromSourceDB];

                                $data_json      = json_encode(['product' => $productMetafieldsCreated], JSON_HEX_TAG);
                                $metafields     = self::curlRequest($productURL . ".json", $data_json, "PUT");
                                $data_insert    = json_encode($metafields, JSON_HEX_TAG);
                                DB::connection('mysql2')->table('shopify_log_details')->insert([
                                    'purl'          => $productURL . ".json",
                                    'method'        => 'Update METAS - Update',
                                    'json_data'     => $data_json,
                                    'response_data' => $data_insert,
                                ]);
                                self::getMetaFields($productId);
                                unset($preparedFromSourceDBMetafields['metafields'][$ikey]);
                                $flgUpdate = 1;
                            }
                        }
                    }
                }

                // post method
                // for insert meta on store and db
                if (!empty($preparedFromSourceDBMetafields['metafields'])) {
                    foreach ($preparedFromSourceDBMetafields['metafields'] as $ikey => $preparedFromSourceDB) {

                        $data_json      = json_encode(['metafield' => $preparedFromSourceDB], JSON_HEX_TAG);
                        $metafields     = self::curlRequest($productURL . "/metafields.json", $data_json, "POST");
                        $data_insert    = json_encode($metafields, JSON_HEX_TAG);
                        DB::connection('mysql2')->table('shopify_log_details')->insert([
                            'purl'          => $productURL . ".json",
                            'method'        => 'Insert METAS - POST',
                            'json_data'     => $data_json,
                            'response_data' => $data_insert,
                        ]);
                        self::getMetaFields($productId);
                        unset($preparedFromSourceDBMetafields['metafields'][$ikey]);
                        $flgUpdate = 1;
                    }
                }
            } 
            // image
            $imageids = explode(",", $shopifyProductOldRow->image_ids);
            if (!empty($imageids)) {
                $imageids = array_filter($imageids);
            }

            $productImages = self::productImages($sku, $product);
            if (isset($productImages['images']) && count($imageids) != count($productImages['images'])) {

                if (count($imageids) <= 0 && !empty($productImages['images'])) {
                    $purl                   = $productURL . ".json";
                    $productArray           = self::curlRequest($purl, "", "GET");
                    $shopifyProductOldRow   = self::imageUpdateToDatabase($productArray, 1, $productId);
                }

                if ((!empty($shopifyProductOldRow) || $shopifyProductOldRow != "") && isset($shopifyProductOldRow->image_ids)) {
                    $imageids = explode(",", $shopifyProductOldRow->image_ids);
                    if (!empty($imageids)) {
                        $imageids = array_filter($imageids);
                    }
                }


                if (count($imageids) > 0) {
                    foreach ($productImages['images'] as $imgKey => $img) {
                        if (!self::imageExists($img['src'])) {
                            unset($productImages['images']);
                        }
                    }
                    if (isset($productImages['images']) && count($imageids) != count($productImages['images'])) {
                        foreach ($imageids as $pid) {
                            $pdeleteurl = $productURL . "/images/" . $pid . ".json";

                            $imageResult = self::curlRequest($pdeleteurl, "", "DELETE");
                            $data_insert = json_encode($imageResult, JSON_HEX_TAG);
                            DB::connection('mysql2')->table('shopify_log_details')->insert([
                                'purl'          => $pdeleteurl,
                                'method'        => 'DELETE Images',
                                'json_data'     => json_encode($pid, JSON_HEX_TAG),
                                'response_data' => $data_insert,
                            ]);
                        }
                        self::imageDeleteFromDatabase($productId);

                        $productMetafieldsCreated           = [];
                        $productMetafieldsCreated['id']     = $productId;
                        $productMetafieldsCreated['images'] = $productImages['images'];
                        $data_json                          = json_encode(['product' => $productMetafieldsCreated], JSON_HEX_TAG);
                        $image_update_url                   = $productURL . ".json";
                        $pImages                            = self::curlRequest($image_update_url, $data_json, "PUT");
                        $data_insert                        = json_encode($pImages, JSON_HEX_TAG);
                        DB::connection('mysql2')->table('shopify_log_details')->insert([
                            'purl'          => $image_update_url,
                            'method'        => 'Image updated while',
                            'json_data'     => $data_json,
                            'response_data' => $data_insert,
                        ]);
                        self::imageUpdateToDatabase($pImages, 0, $productId);
                        $flgImage = 1;
                    }
                }
            }
            if ($flgUpdate == 1 || $flgInsert == 1 || $flgDelete == 1 || $flgImage == 1) { 
                DB::connection('mysql2')->table('shopify_product_report')->insert([
                    'action'                => 'UPDATE',
                    'shopify_product_id'    => $productId,
                    'created_at'            => date('Y-m-d'),
                    'cron_name'             => $cronName,
                ]);
            } 
        } catch (\Exception $e) {
            $this->errorLog($e, $cronName);
            report($e);
            Log::error("Edit error - updateProduct()");
        }
    }

    /**
     * @param $shopifyProductOldRow
     * @param $status
     * @param $title
     */
    public static function updateProductTags($shopifyProductOldRow, $status, $title)
    {

        $sku            = $shopifyProductOldRow->product_sku;
        // $product_type   = self::getProductTypeId($shopifyProductOldRow->product_type);
        $productId      = $shopifyProductOldRow->product_id;
        $url            = self::getStoreUrl();
        $productURL     = $url . "products/$productId";
        $purl           = $productURL . ".json?fields=id,tags";
        $productArray   = self::curlRequest($purl, "", "GET");
        if(isset($productArray['product'])){
            $productMetafieldsCreated           = [];
            $productMetafieldsCreated['id']     = $productId;
            $tags                               = $productArray['product']['tags'].",YGroup_".$sku;
            $productMetafieldsCreated['tags']   = $tags;
            $data_json                          = json_encode(['product' => $productMetafieldsCreated], JSON_HEX_TAG);
            $tags_update_url                    = $productURL . ".json";
            $pImages                            = self::curlRequest($tags_update_url, $data_json, "PUT");
            $data_insert                        = json_encode($pImages, JSON_HEX_TAG);
            DB::connection('mysql2')->table('shopify_log_details')->insert([
                'purl'          => $tags_update_url,
                'method'        => 'tags updated while',
                'json_data'     => $data_json,
                'response_data' => $data_insert,
            ]);
            DB::connection('mysql2')->table('shopify_products_new')->where('product_id',$productId)->update([
                'tags' => $tags,
            ]);
        }

    }

    /**
     * @param $uri
     * @return bool
     */
    private static function imageExists($uri)
    {
        $ch = curl_init($uri);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        return $code == 200;
    }

    /**
     * @param $productArray
     * @param $flg
     * @param $productId
     * @return \Illuminate\Database\Query\Builder|string
     */
    public function imageUpdateToDatabase($productArray, $flg, $productId)
    {
        $shopifyProductOldRow = "";
        if (isset($productArray['product']['images']) && !empty($productArray['product']['images'])) {
            $imagesrc = $imageIds = [];
            foreach ($productArray['product']['images'] as $image) {
                list($filename, $Vtime) = explode("?", basename($image['src']));
                $imagesrc[] = $filename;
                $imageIds[] = $image['id'];
            }
            DB::connection('mysql2')->table('shopify_products_new')
            ->where(['product_id' => $productId])->update([
                'image_ids'     => !empty($imageIds) ? implode(",", $imageIds) : "",
                'images'        => !empty($imagesrc) ? implode(",", $imagesrc) : ""
            ]);
            if ($flg == 1) {
                $shopifyProductOldRow = DB::connection('mysql2')->table('shopify_products_new')
                    ->where(['product_id' => $productId]);
            }
        }
        return $shopifyProductOldRow;
    }

    /**
     * @param $productId
     * @return \Illuminate\Database\Query\Builder|string
     */
    public function imageDeleteFromDatabase($productId)
    {
        $results = DB::connection('mysql2')->table('shopify_products_new')->where(['product_id' => $productId])->update([
            'image_ids'     => "",
            'images'        => ""
        ]);
        return $results;
    }

    /**
     * @param $productId
     * @return \Illuminate\Support\Collection
     */
    public function isMetaFieldsExist($productId)
    {
        return DB::connection('mysql2')->table('shopify_products_metafiels')
            ->select('product_id', 'namespace', 'key', 'value', 'value_type')
            ->where(['product_id' => $productId])->get();

    }

    /**
     * @param $productPartNumber
     * @return int
     */
    public function filterType($productPartNumber)
    {
        $return = 0;
        if ($productPartNumber->partType == OIL_PAN_DRAIN_PLUG_GASKET) {
            $return = 55;
        } else if ($productPartNumber->partType == OIL_PAN_DRAIN_PLUG) {
            $return = 50;
        }
        return $return;
    }

    /**
     * @param $url
     * @param $data_json
     * @param $method
     * @return mixed
     */
    public function curlRequest($url, $data_json, $method)
    {
        try {
            // dd("stop");
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            if ($method == "POST") {
                curl_setopt($ch, CURLOPT_POST, true);
            } else if ($method == "GET") {
                curl_setopt($ch, CURLOPT_POST, false);
            }

            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Accept: application/json', 'Content-Type: application/json'));
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            if ($method == "POST" || $method == "PUT") {
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
            }

            $output         = curl_exec($ch);
            $statusCode     = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            if ($statusCode == 429) {
                DB::connection('mysql2')->table('error_log')->insert([
                    'code'      => 429,
                    'cron_name' => $this->cronName,
                ]);
                Log::alert($statusCode);
                Log::alert($url);
                sleep(ONE);

                return self::curlRequest($url, $data_json, $method);
            }
            if (curl_errno($ch)) {
                Log::info(curl_error($ch));
                $logs = [
                    'result'    => "",
                    'url'       => $url,
                    'data'      => json_decode($data_json, true),
                    'method'    => $method,
                    'error'     => "curlRequest"
                ];
                DB::connection('mysql2')->table('error_log')->insert([
                    'message'   => $method,
                    'details'   => curl_error($ch),
                    'code'      => curl_errno($ch) ?? null,
                    'cron_name' => $this->cronName  ?? null,
                ]);
                Log::error(json_encode($logs));
                sleep(ONE);
                return self::curlRequest($url, $data_json, $method);
            }
            curl_close($ch);
            $result = json_decode($output, true);
            if (isset($result['errors'])) {
                $logs = [
                    'result'    => $result,
                    'url'       => $url,
                    'data'      => json_decode($data_json, true),
                    'method'    => $method,
                    'error'     => 1
                ];
                if (isset($result['errors'][ZERO]['message']) && $result['errors'][ZERO]['message'] == "You have sent too many requests. Try again soon.") {
                    sleep(ONE);
                    return self::curlRequest($url, $data_json, $method);
                }
                Log::error(json_encode($logs));

            } else {
                if (isset($result['product'])) {
                    // Log::info("product id => " . $result['product']['id']);
                }
            }
            return $result;

        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
            if ($e->getCode() == 429) {
                Log::alert($e->getCode());
                Log::alert($url);
                sleep(ONE);
                return self::curlRequest($url, $data_json, $method);
            }
        }
    }

    /**
     * @param $productPartNumber
     * @param int $isEdit
     * @return string
     */ 
    public function getStoreUrl($type = null)
    {
        if($type == "NextYearSEO"){
            $url = "https://" . env('SHOPIFY_KEY_1') . ":" . env('SHOPIFY_SECREAT_1') . "@" . env('SHOPIFY_STORE') . ".myshopify.com/admin/" . env('SHOPIFY_API_VERSION') . "/";
        } else {
            $url = "https://" . env('SHOPIFY_KEY_1') . ":" . env('SHOPIFY_SECREAT_1') . "@" . env('SHOPIFY_STORE') . ".myshopify.com/admin/" . env('SHOPIFY_API_VERSION') . "/";
        }
        return $url;
    }

    /**
     * @param $productId
     * @return array
     */
    public function getMetaFields($productId)
    {
        try {
            $url            = self::getStoreUrl();
            $url            = $url . "products/$productId/metafields.json";
            $metafields     = self::curlRequest($url, "", "GET");
            $data_insert    = json_encode($metafields, JSON_HEX_TAG);

            DB::connection('mysql2')->table('shopify_log_details')->insert([
                'purl'          => $url,
                'method'        => 'GET METAS',
                'response_data' => $data_insert,
            ]);
            if ($metafields) {
                if (!array_key_exists('errors', $metafields) && !array_key_exists('error', $metafields)) {
                    foreach ($metafields as $m => $mvalue) {
                        foreach ($mvalue as $k => $val) {
                            self::createMetaFields($val, $productId);
                        }
                    }
                }
            }
            return $metafields;
        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
            Log::error("get meta and create meta if not exist");
            //self::emailTriggerOnError($e->getMessage());
        }
    }

    /**
     * @param $val
     * @param $productId
     * @return mixed
     */
    public function createMetaFields($val, $productId)
    {
        try {
            return DB::connection('mysql2')->table('shopify_products_metafiels')->updateOrInsert(
                [
                    'product_id'    => $productId ?? null,
                    NAMESPACES      => $val[NAMESPACES] ?? null,
                    KEY             => $val[KEY] ?? null,
                ],[
                    'product_id'    => $productId,
                    'metafield_id'  => $val[ID],
                    NAMESPACES      => $val[NAMESPACES],
                    KEY             => $val[KEY],
                    VALUE           => $val[VALUE],
                ]);
        } catch (\Exception $e) {
            Log::error("insert metafield => " . json_encode([
                    'product_id'    => $productId,
                    'metafield_id'  => $val[ID],
                    NAMESPACES      => $val[NAMESPACES],
                    KEY             => $val[KEY],
                    VALUE           => $val[VALUE],
                ]));
            report($e);
        }
    }

    /**
     *  Email trigger
     */
    public function emailTrigger($subject = "")
    {
        try {
            $products = DB::connection('mysql2')
                ->table('shopify_products_new')
                ->select('product_title', 'product_id', 'product_sku')
                ->where('emailSend', ZERO)
                ->get();
            $emailPartNumber = $productId = $sku = [];
            foreach ($products as $product) {
                $emailPartNumber[]  = $product->product_title;
                $productId[]        = $product->product_id;
                $sku[]              = $product->product_sku;
            }
            if (!empty($emailPartNumber)) {
                $data['partNumber'] = implode(',', $emailPartNumber);
                $data['newSku']     = implode(',', array_unique($sku));

                // send email
                self::mailDispatch("", $data, $subject);
                DB::connection('mysql2')->table('shopify_products_new')
                    ->whereIn('product_id', $productId)
                    ->update(['emailSend' => ONE]);
            }
        } catch (\Exception $e) {
            $this->errorLog($e);
            report($e);
        }
    }

    public function mailDispatch($is_error, $mailContent, $subject = "")
    {
        if ($subject == "") {
            $subject = 'Pureflow product create script success';
        }
         
        if ($is_error == "error") {
            $to = config("app.email_ids.create_product_script_error");
            //$to = array_push($to,'ayushawasthi@mobikasa.com');
            self::send('emails.create_product_error', $to, 'Pureflow Product create Script error', $mailContent);
        } else { 
            $to = config("app.email_ids.create_product_script_success");
            self::send('emails.new_product', $to, $subject, $mailContent);
        }
    }

    public function send($view, $to, $subject, $mailContent)
    {
        try {
            Mail::send(['html' => $view], $mailContent, function ($message) use ($to, $subject) {
                $message->to($to)
                    ->subject($subject);
                $message->from(env('MAIL_FROM_ADDRESS'), 'Pureflow');
            });

        } catch (\Exception $e) {
            $this->errorLog($e);
            Log::error("Mail send failed ==> send()");
            Log::error(json_encode($mailContent));
            report($e);
        }
    }

    /**
     * @param $errorData
     */
    public function emailTriggerOnError($errorData)
    {
        try {
            if (!empty($errorData)) {
                self::mailDispatch("error", ['errorData' => $errorData]);
            }
        } catch (\Exception $e) {
            $this->errorLog($e);
            Log::error("Mail send failed => emailTriggerOnError()");
            report($e);
        }
    }

    public function updateTitleHandle($applications, $productPartNumber)
    {
        try {
            foreach ($applications as $key => $part) {
                $arrayPart      = (array)$part;
                $productPart    = self::getProductType($part);

                $shopifyProductNew = DB::connection('mysql2')->table('shopify_products_new')
                    ->select('product_id', 'id')
                    ->where([
                        YEAR        => $productPart[YEAR],
                        MAKE        => $productPart[MAKE],
                        MODEL       => $productPart[MODEL],
                        PRODUCT_SKU => $arrayPart['partnumber']
                    ])->whereNull('part_type')->get()->toArray();
                if (count($shopifyProductNew) > ZERO) {
                    $productId = $shopifyProductNew[ZERO]->product_id;
                    if (!isset($arrayPart['partnumber'])) {
                        Log::error(json_encode($part));
                        die;
                    }
                    $sku                = $arrayPart['partnumber'];
                    $product            = [];
                    $product['tags']    = "";
                    $sizeVariant        = self::getPackages($sku);
                    $product            = self::productImages($sku, $product); // set image url
                    $product            = self::basicProductDetails($product, $productPart, $sku, [], $part, $productPartNumber); // set product basic keys
                    $newProduct['product'] = [
                        'id'        => $productId,
                        'title'     => $product['title'],
                        'handle'    => $product['handle'],
                    ];
                    $data_json          = json_encode($newProduct, JSON_HEX_TAG);
                    $url                = self::getStoreUrl();
                    $url                = $url . "products/$productId.json";
                    $shopifyResponse    = self::curlRequest($url, $data_json, "PUT");
                    $data_insert        = json_encode($shopifyResponse, JSON_HEX_TAG);
                    DB::connection('mysql2')->table('shopify_log_details')->insert([
                        'purl'          => $url,
                        'method'        => 'NEW PRODUCT CREATION',
                        'json_data'     => $data_json,
                        'response_data' => $data_insert,
                    ]);

                    if (!array_key_exists('errors', $shopifyResponse) && !array_key_exists('error', $shopifyResponse)) {
                        DB::connection('mysql2')->table('shopify_products_new')
                            ->select('product_id', 'id')
                            ->where([
                                'id' => $shopifyProductNew[ZERO]->id,
                            ])->update([
                                'product_title' => $shopifyResponse['product']['title'],
                                'handle'        => $shopifyResponse['product']['handle']
                            ]);
                    } else {
                        //dd("Product title handle update",$data_insert);
                        Log::info("Product title handle update");
                        Log::error($data_insert);
                    }
                }
            }
        } catch (\Exception $e) {
            $this->errorLog($e);
            Log::error("Error : updateTitleHandle()");
            report($e);
        }
    }

    public function seoProductUpdate($product, $part, $productPart, $shopifySeoProduct, $productPartNumber)
    {
        $product['id'] = $shopifySeoProduct->product_id;
        if(isset($shopifySeoProduct->variant_id)){
            $product['variants'][ZERO]['id'] = $shopifySeoProduct->variant_id;
        }

       //$metafields = $product['metafields'];
       //$images = $product['images'];
       //unset($product['metafields']);
        unset($product['images']);
        return $product;
    }

    public function updatePublished_atDate($product, $flg = ZERO)
    { 
        $url            = self::getStoreUrl();
        $productId      = $product->product_id;
        $productURL     = $url . "products/$productId.json?fields=id,title,handle,published_at";
        $product_data   = self::curlRequest($productURL, "", "GET");
        DB::connection('mysql2')->table('shopify_log_details')->insert([
            'purl'          => $productURL,
            'method'        => 'published at date',
            'json_data'     => json_encode($product),
            'response_data' => json_encode($product_data),
        ]);
        if ($product_data["product"]["published_at"] != null) {
            DB::connection('mysql2')->table('shopify_products_new')->where(['product_id' => $product_data["product"]["id"]])->update([
                'published_at'      => $product_data["product"]["published_at"],
                'published_picked'  => 1,
            ]);
            Log::info("Published_at date updated for  =>" . $product_data["product"]["id"]);
            DB::connection('mysql2')->table('shopify_product_report')->insert([
                'action'                => 'PUBLISHED_AT_UPDATE',
                'shopify_product_id'    => $product_data["product"]["id"],
                'cron_name'             => 'product-published-at-date-updated',
                'created_at'            => date('Y-m-d'),
            ]);
        }
        DB::connection('mysql2')->table('shopify_products_new')->where(['product_id' => $product_data["product"]["id"]])->update([
            'published_picked' => 1,
        ]);
    }

    public function errorLog($e,$cronname = null)
    {
        if(empty($cronname)){
            $cronname = $this->cronName;
        }
        $data = [
            'message'   => $e->getMessage(),
            'details'   => $e,
            'code'      => $e->getCode() ?? null,
            'cron_name' => $cronname ?? null,
        ]; 

        DB::connection('mysql2')->table('error_logs')->insert($data);
 
        $recipients = config("app.email_ids.error_detail");
        Mail::to($recipients)->send(new ErrorDetailMail($data));
    }

    public function createNextYearSEOProducts($oldSEOProduct)
    {
        $nextYear           = date('Y') + 1;
        $currentYear        = date('Y');
        $shopifyProductId   = $oldSEOProduct->product_id;

        // Check if next year's product already exists
        $nextProductsCount = DB::connection('mysql2')->table('shopify_products_new')
        ->where([ 
            'year'          => $nextYear,
            'make'          => $oldSEOProduct->make,
            'model'         => $oldSEOProduct->model,
            'product_sku'   => $oldSEOProduct->product_sku,
        ])->count();

        if ($nextProductsCount > 0) {
            return; // Next year's product already exists, no need to create
        }

        // Retrieve product details from Shopify
        $url        = $this->getStoreUrl("NextYearSEO");
        $productURL = $url . "products/$shopifyProductId.json";
        $getProduct = $this->curlRequest($productURL, "", "GET");

        if (array_key_exists('errors', $getProduct) || array_key_exists('error', $getProduct)) {
            return; // Handle error when retrieving product details
        }

        // Process product data
        $newProduct = $this->processProductData($getProduct, $nextYear, $currentYear);

        // Insert new product into Shopify
        $this->insertNewProduct($newProduct, $oldSEOProduct);
    }

    private function processProductData($productData, $nextYear, $currentYear)
    {
        $newProduct         = [];
        $product            = $productData['product'];
        // Modify product details for next year
        $product['title']   = str_replace($currentYear, $nextYear, $product['title']);
        $product['handle']  = str_replace($currentYear, $nextYear, $product['handle']);
        $tags               = array_map("trim", explode(",", $product['tags']));
        foreach ($tags as $k => &$tag) {
            if ($tag == "Year_" . $currentYear || $tag == "year_" . $currentYear) {
                unset($tag);
                $tags[$k] = "Year_" . $nextYear;
            }
        }
        array_push($tags, 'seo_product');
        $product['tags']        = implode(",", $tags);
        $product['published']   = 'false';

        // Modify variant details
        foreach ($product['variants'] as &$variant) {
            unset($variant['id']);
            unset($variant['product_id']);
            $variant = [
                'title'                 => $variant['title'],
                'sku'                   => $variant['sku'],
                'weight'                => $variant['weight'],
                'price'                 => $variant['price'],
                'inventory_management'  => 'shopify',
                'inventory_quantity'    => $variant['inventory_quantity'],
            ];
        }

        // Modify image details
        foreach ($product['images'] as &$image) {
            unset($image['id']);
            unset($image['product_id']);
            $image = [
                'src'       => $image['src'],
                'position'  => $image['position'],
                'alt'       => $image['alt'],
                'width'     => $image['width'],
                'height'    => $image['height'],
            ];
        }

        // Modify metafield details
        foreach ($product['metafields'] as &$metafield) {
            unset($metafield['id']);
            unset($metafield['owner_id']);
            $metafield = [
                'namespace' => $metafield['namespace'],
                'key'       => $metafield['key'],
                'value'     => $metafield['value'],
                'type'      => $metafield['type'],
            ];
        }

        $newProduct['product'] = $product;

        return $newProduct;
    }

    private function insertNewProduct($product, $oldSEOProduct)
    {

        $sku                    = $oldSEOProduct->partNumber;
        $product                = [];
        $sizeVariant            = self::getPackages($sku);
        $oemNumber              = self::getOEMNumber($sku);
        $technicalDetails       = self::getTechnicalDetail($sizeVariant, $sku, $oemNumber);
        $productPart            = json_decode(json_encode($oldSEOProduct), true);


        $url                = $this->getStoreUrl("NextYearSEO");
        $productURL         = $url . "products.json";
        $postJson           = json_encode(['product' => $product['product']]);
        $shopifyResponse    = $this->curlRequest($productURL, $postJson, "POST");

        $data_insert = json_encode($shopifyResponse, JSON_HEX_TAG);
        DB::connection('mysql2')->table('shopify_log_details')->insert([
            'purl'          => $url,
            'method'        => 'NEXT YEAR SEO PRODUCT CREATION',
            'json_data'     => $postJson,
            'response_data' => $data_insert,
        ]);

        if (!array_key_exists('errors', $shopifyResponse) && !array_key_exists('error', $shopifyResponse)) {
            DB::connection('mysql2')->table('shopify_product_report')->insert([
                'action'                => 'SEO_PRODUCT_CREATE',
                'shopify_product_id'    => $shopifyResponse['product']['id'],
                'cron_name'             => "seo-next-year",
                'created_at'            => date('Y-m-d'),
            ]);
            $imagesrc = $image_ids = [];
            foreach ($shopifyResponse['product']['images'] as $image) {
                list($filename, $Vtime) = explode("?", basename($image['src']));
                $imagesrc[]             = $filename;
                $image_ids[]            = $image['id'];
            }

            $shopifyResponse['product']['src']      = implode(',', $imagesrc);
            $shopifyResponse['product']['img_ids']  = implode(',', $image_ids);
            $tagCheck                               = explode(',', $shopifyResponse['product']['tags']);
            $trimmed_tags                           = array_map('trim', $tagCheck);
            $product['next_year_copy']              = ONE;
            $product['product_type']                = $shopifyResponse['product']['product_type'];

            $productNumberPart              = new \stdClass();
            $productNumberPart->part_id     = $sku;
            $productNumberPart->partNumber  = $sku;

            $this->createProduct($shopifyResponse, $sku, $technicalDetails, $trimmed_tags, $productPart, $product, $productNumberPart);
            $this->getMetaFields($shopifyResponse['product']['id']);
        }
    }
}
 