<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\ProductController; 
use Illuminate\Support\Facades\{DB, Log};

class WebhooksController extends Controller
{
    public function handleCreateWebhook(Request $request)
    { 
        $webhookContent     =   $request->getContent();
        $productController  =   new ProductController();
        try {
            $webhookArray = json_decode($webhookContent, true);
            if ($webhookArray) {
                $tags = $webhookArray['tags'] ?? '';
                if (!empty($tags)) {
                    $tagArray   = explode(',', $tags);
                    $del_val    = 'Category_Default Category/default';
                    foreach ($tagArray as $key => $value) {
                        if (trim($value) == $del_val) {
                            unset($tagArray[$key]);
                        }
                    }
                    $tagsJson = json_encode($tagArray, JSON_HEX_APOS);

                    DB::beginTransaction();
                    $dataid = DB::table('shopify_products')->insertGetId([
                        'product_id'            => $webhookArray['id'],
                        'product_title'         => $webhookArray['title'],
                        'handle'                => $webhookArray['handle'],
                        'product_sku'           => $webhookArray['variants'][0]['sku'],
                        'tags'                  => $tagsJson,
                        'product_created_at'    => $webhookArray['created_at']
                    ]);

                    if (!$dataid) {
                        DB::rollBack();
                        return response()->json(['error' => 'Failed to insert data into shopify_products table'], 500);
                    }

                    DB::commit();
                }
            }
        } catch (\Exception $ex) {
            $productController->errorLog($ex, "Pureflow create webhook");
            \Log::error($ex->getMessage());
            return response()->json(['error' => 'An error occurred while processing the webhook'], 500);
        }

        return response()->json(['success' => true]);
    }


    public function handleDeleteWebhook(Request $request)
    { 
        $webhookContent = $request->getContent();

        try {
            $webhookArray = json_decode($webhookContent, true);
            if ($webhookArray) {
                DB::beginTransaction();
                $dataid = DB::table('shopifydata')->insertGetId([
                    'json_data' => json_encode($webhookArray, JSON_UNESCAPED_SLASHES),
                    'event'     => 'product delete'
                ]);

                if ($dataid) {
                    DB::table('shopify_products_new')->where('product_id', $webhookArray['id'])->delete();
                    DB::table('shopify_products')->where('product_id', $webhookArray['id'])->delete();
                    DB::table('shopify_products_qualifier')->where('product_id', $webhookArray['id'])->delete();
                }

                DB::commit();
            }
        } catch (\Exception $ex) {
            $productController  =   new ProductController();
            $productController->errorLog($ex, "Pureflow Delete Webhook");
            \Log::error($ex->getMessage());
            DB::rollBack();
            return response()->json(['error' => 'An error occurred while processing the webhook'], 500);
        }

        return response()->json(['success' => true]);
    }
}
