<?php

namespace App\Mail;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;
use App\Exports\CreatedProductsReportExport;
use Maatwebsite\Excel\Facades\Excel;  
use Illuminate\Support\Facades\Storage; 

class CreatedProductsReportMail extends Mailable
{
    use Queueable, SerializesModels;

    public $emailType;

    public function __construct($emailType = null)
    {
        $this->emailType = $emailType;
    }

    public function build()
    {
        $fileName = 'Newly_created_products_list.xlsx';

        // Generate Excel file
        $excelFile = Excel::download(new CreatedProductsReportExport($this->emailType), $fileName, \Maatwebsite\Excel\Excel::XLSX);
        // Store Excel file temporarily
        $tempFilePath = 'temp/' . $fileName;
        Storage::put($tempFilePath, $excelFile->getFile()->getContent());

        // Attach Excel file to email
        return $this->view('emails.created_products_report')
            ->attach(storage_path('app/' . $tempFilePath), [
                'as'    => $fileName,
                'mime'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            ]);
 
        // ->with(['data' => $this->object]); 
    }

    /**
     * Get the message envelope.
     */
    public function envelope(): Envelope
    {
        if ($this->emailType == "INTERNAL") { 
            return new Envelope(
                subject: 'Product Created Daily Internal Report - Pureflow',
            ); 
        }
        return new Envelope(
            subject: 'Product Created Daily Report - Pureflow',
        );


    }

    /**
     * Get the attachments for the message.
     *
     * @return array<int, \Illuminate\Mail\Mailables\Attachment>
     */
    public function attachments(): array
    {    
        return [];
    }
}
