<?php

namespace App\Mail;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;
use App\Exports\DailyErrorLogsReportExport;
use Maatwebsite\Excel\Facades\Excel; 
use Illuminate\Support\Facades\{DB, Log, Storage};

class DailyErrorLogsReportMail extends Mailable
{
    use Queueable, SerializesModels;

    /**
     * Create a new message instance.
     */
    public $message;
    public $logsCount;

    public function __construct()
    {
        $date = date('Y-m-d', strtotime("-1 day"));

        $this->logsCount = DB::connection('mysql2')->table('error_logs')
        ->where('created_at', '>=', $date . " 00:00:00")
        ->where('created_at', '<=', $date . " 23:59:00")->count();

    }

    /**
     * Get the message envelope.
     */
    public function envelope(): Envelope
    {
        return new Envelope(
            subject: 'Daily Error Logs Report - Pureflow',
        );
    }

    public function build()
    {
        if ($this->logsCount > 0) {
            $fileName = 'Logs.xlsx';
            // Generate Excel file
            $excelFile = Excel::download(new DailyErrorLogsReportExport(), $fileName, \Maatwebsite\Excel\Excel::XLSX);
            // Store Excel file temporarily
            $tempFilePath = 'temp/' . $fileName;
            Storage::put($tempFilePath, $excelFile->getFile()->getContent());

            // Attach Excel file to email 
            return $this->view('emails.daily_error_logs_report') 
                ->attach(storage_path('app/' . $tempFilePath), [
                    'as'    => $fileName,
                    'mime'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                ]);
        } 
        return $this->view('emails.no_error_logs_report')->with('message', $this->message);
    }

    /**
     * Get the message content definition.
     */
    // public function content(): Content
    // {
    //     return new Content(
    //         view: 'view.name',
    //     );
    // }

    /**
     * Get the attachments for the message.
     *
     * @return array<int, \Illuminate\Mail\Mailables\Attachment>
     */
    public function attachments(): array
    {
        return [];
    }
}
