<?php

namespace App\Mail;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;
use App\Exports\NotPublishedProductsReportToSalesTeamExport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Storage; 

class NotPublishedProductsReportToSalesTeamMail extends Mailable
{
    use Queueable, SerializesModels;

    /**
     * Create a new message instance.
     */
    // public function __construct()
    // {
    //     //
    // }

    /**
     * Get the message envelope.
     */
    public function envelope(): Envelope
    {
        return new Envelope(
            subject: 'Sales Channel Report (Not Published Products) - Pureflow',
        );
    }

    public function build()
    {
        $fileName = 'Unpublished_Product_Report.xlsx';
        
        // Generate Excel file
        $excelFile = Excel::download(new NotPublishedProductsReportToSalesTeamExport(), $fileName, \Maatwebsite\Excel\Excel::XLSX);
        
        // Store Excel file temporarily
        $tempFilePath = 'temp/' . $fileName;
        Storage::put($tempFilePath, $excelFile->getFile()->getContent());

        // Attach Excel file to email 
        return $this->view('emails.unpublished_products_report')
        ->attach(storage_path('app/' . $tempFilePath), [
            'as'    => $fileName,
            'mime'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        ]);
    }

    /**
     * Get the attachments for the message.
     *
     * @return array<int, \Illuminate\Mail\Mailables\Attachment>
     */
    public function attachments(): array
    {
        return [];
    }
}
